"""Tests for CMFNotification installation.

$Id: testInstallation.py 55153 2007-12-09 12:42:44Z dbaty $
"""

from zope.component import getUtility
from AccessControl.PermissionRole import rolesForPermissionOn

from plone.portlets.interfaces import IPortletManager

from Products.CMFCore.utils import getToolByName

from Products.CMFNotification import NotificationTool
from Products.CMFNotification.permissions import SUBSCRIBE_PERMISSION
from Products.CMFNotification.tests.plonetestbrowser import Browser
from Products.CMFNotification.tests.utils import CMFNotificationTestCase


class TestInstallation(CMFNotificationTestCase):
    """Make sure that the product is properly installed."""

    def afterSetUp(self):
        pass


    def testToolIsThere(self):
        portal = self.portal
        self.failUnless(NotificationTool.ID in portal.objectIds())


    def testPortletCanBeAdded(self):
        base_url = self.portal.absolute_url()
        for name in ('plone.leftcolumn', 'plone.rightcolumn'):
            manager = getUtility(IPortletManager,
                                 name=name,
                                 context=self.portal)
            titles = [p.title for p in manager.getAddablePortletTypes()]
            self.failUnless('Subscription portlet' in titles)


    def testPermissionHasBeenSet(self):
        roles = set(rolesForPermissionOn(SUBSCRIBE_PERMISSION, self.portal))
        self.failUnlessEqual(roles, set(('Manager', 'Member')))


    def testConfigletHasBeenAdded(self):
        cptool = getToolByName(self.portal, 'portal_controlpanel')
        configlets = [c.getId() for c in cptool.listActions()]
        self.failUnless('cmfnotification_configuration' in configlets)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    return suite
